#include <iostream>
#include <memory>
using namespace std;

#include "cppauto.h"

/*

A class that has to call something back

*/
class Caller
{
public:
	/*
	
	Note that auto_ptr<> is used here to ensure that the MyCallback
	object is destroyed properly.  If you don't like to use the STL or
	prefer to trust yourself to delete it later, use this instead:
	
	static void CallMeBack( MyCallback* cb )
	
	*/
	static void CallMeBack( auto_ptr< MyCallback > cb )
	{
		cb->Call( 1, 2, 3 );
	}
};


/*

An object that wants to be called back

*/
class Receiver
{
public:
	/*
	
	The method we want to call back.
	
	*/
	void DoStuff( int a, int b, int c )
	{
		cout << "Caught callback, a = " << a << ", b = " << b << ", c = " << c << endl;
	}
	
	/*
	
	Method to do the testing.  Makes a delegate, passes it to Caller::CallMeBack,
	and gets called back.
	
	*/
	void Test()
	{
		/*
		
		As above, we use auto_ptr<> to transfer ownership of the pointer to
		the right place and to ensure that our MyCallback object gets deleted.
		If you prefer to do your deallocation manually, change the definition
		below to the following:
		
		MyCallback* cb = new_MyCallback( this, &Receiver::DoStuff );
		
		*/
		auto_ptr< MyCallback > cb( new_MyCallback( this, &Receiver::DoStuff ) );

		/*
		
		Send our delegate object off so we can be called back.
		
		*/
		Caller::CallMeBack( cb );
		
		/*
		
		If you don't want to use auto_ptr<>, don't forget to deallocate cb:
		
		delete cb;
		
		*/
	}
};

// Stub main() to run Receiver::Test()
int main( void )
{
	/*
	
	Instantiate a Receiver and run the test ...
	
	*/
	Receiver testReceiver;
	testReceiver.Test();
}
